@@ -10,7 +10,7 @@
 
 """Injector - Python dependency injection framework, inspired by Guice
 
-See http://pypi.python.org/pypi/injector for documentation.
+See https://github.com/alecthomas/injector for documentation.
 
 :copyright: (c) 2012 by Alec Thomas
 :license: BSD
@@ -19,17 +19,28 @@ See http://pypi.python.org/pypi/injector for documentation.
 import itertools
 import functools
 import inspect
+import logging
 import sys
 import types
 import threading
 from abc import ABCMeta, abstractmethod
 from collections import namedtuple
 
+try:
+    NullHandler = logging.NullHandler
+except AttributeError:
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
 
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.6.3'
+__version__ = '0.7.0'
 __version_tag__ = ''
 
+log = logging.getLogger(__name__)
+# log.setLevel(logging.WARNING)
+log.addHandler(NullHandler())
+
 
 def synchronized(lock):
     def outside_wrapper(function):
@@ -202,10 +213,10 @@ class BindingKey(tuple):
         return self[1]
 
 
-BindingBase = namedtuple("BindingBase", 'interface annotation provider scope')
+_BindingBase = namedtuple('_BindingBase', 'interface annotation provider scope')
 
 
-class Binding(BindingBase):
+class Binding(_BindingBase):
     """A binding from an (interface, annotation) to a provider in a scope."""
 
 
@@ -255,7 +266,7 @@ class Binder(object):
         A multi-binding maps from a key to a sequence, where each element in
         the sequence is provided separately.
 
-        :param interface: Interface or :func:`Key` to bind.
+        :param interface: :func:`MappingKey` or :func:`SequenceKey` to bind to.
         :param to: Instance, class to bind to, or an explicit :class:`Provider`
                 subclass. Must provide a sequence.
         :param annotation: Optional global annotation of interface.
@@ -382,6 +393,9 @@ class ScopeDecorator(object):
 
     def __call__(self, cls):
         cls.__scope__ = self.scope
+        binding = getattr(cls, '__binding__', None)
+        if binding:
+            setattr(cls, '__binding__', Binding(*binding))
         return cls
 
     def __repr__(self):
@@ -463,14 +477,6 @@ class Module(object):
                     annotation=binding.annotation,
                     scope=binding.scope,
                     )
-            elif hasattr(function, '__multibinding__'):
-                binding = function.__multibinding__
-                binder.multibind(
-                    binding.interface,
-                    to=types.MethodType(binding.provider, self),
-                    annotation=binding.annotation,
-                    scope=binding.scope,
-                    )
         self.configure(binder)
 
     def configure(self, binder):
@@ -545,7 +551,10 @@ class Injector(object):
         except UnsatisfiedRequirement as e:
             raise Error('%s; scopes must be explicitly bound '
                         'with Binder.bind_scope(scope_cls)' % e)
-        return scope_instance.get(key, binding.provider).get()
+
+        result = scope_instance.get(key, binding.provider).get()
+        log.debug('get(%r, annotation=%r, scope=%r) -> %r', interface, annotation, scope, result)
+        return result
 
     def create_child_injector(self, *args, **kwargs):
         return Injector(*args, parent=self, **kwargs)
@@ -652,27 +661,15 @@ def provides(interface, annotation=None, scope=None, eager=False):
     :param scope: Optional scope of provided value.
     """
     def wrapper(provider):
-        provider.__binding__ = Binding(interface, annotation, provider, scope)
+        scope_ = scope or getattr(provider, '__scope__', getattr(wrapper, '__scope__', None))
+        provider.__binding__ = Binding(interface, annotation, provider, scope_)
         return provider
 
     return wrapper
 
 
 def extends(interface, annotation=None, scope=None):
-    """A decorator for :class:`Module` methods that return sequences of
-    implementations of interface.
-
-    This is a convenient way of declaring a :meth:`Module.multibind` .
-
-    :param interface: Interface to provide.
-    :param annotation: Optional annotation value.
-    :param scope: Optional scope of provided value.
-    """
-    def wrapper(provider):
-        provider.__multibinding__ = Binding(interface, annotation, provider, scope)
-        return provider
-
-    return wrapper
+    raise DeprecationWarning('@extends({}|[]) is deprecated, use @provides and SequenceKey or MappingKey')
 
 
 if hasattr(inspect, 'getfullargspec'):
